'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.originalLoader = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _module = require('module');

var _module2 = _interopRequireDefault(_module);

var _plugins = require('./plugins');

var _mocks = require('./mocks');

var _globals = require('./globals');

var _globals2 = _interopRequireDefault(_globals);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var originalLoader = exports.originalLoader = _module2.default._load;

var patternMatch = function patternMatch(fileName) {
    return function (pattern) {
        if (typeof pattern == 'function') {
            return pattern(fileName);
        }
        return fileName.match(pattern);
    };
};

var testPassby = function testPassby(request, module) {
    var _getScope = (0, _globals2.default)(),
        parentModule = _getScope.parentModule,
        passBy = _getScope.passBy,
        mockedModules = _getScope.mockedModules,
        isolation = _getScope.isolation;

    // was called from test


    if (module === parentModule || module == module.parent) {
        return true;
    }
    // if parent is in the pass list - pass everything
    var fileName = _module2.default._resolveFilename(request, module);
    var m = module;
    while (m) {
        if (!isolation.noAutoPassBy && mockedModules[fileName] || // parent was mocked
        passBy.filter(patternMatch(fileName)).length // parent is in pass list
        ) {
                return true;
            }
        fileName = m.filename;
        m = m.parent;
    }
    return false;
};

function mockResult(name, data) {
    return data;
}

function mockLoader(request, parent, isMain) {
    var _getScope2 = (0, _globals2.default)(),
        parentModule = _getScope2.parentModule,
        mockedModules = _getScope2.mockedModules,
        isolation = _getScope2.isolation;

    var baseRequest = _module2.default._resolveFilename(request, parent);

    if (parent === parentModule) {
        delete _module2.default._cache[baseRequest];
        mockedModules[baseRequest] = true;
    }

    var mock = (0, _mocks.getMock)(baseRequest) || (0, _mocks.getMock)(request);

    if (mock) {
        if ((0, _plugins.shouldMock)(mock, request, parent, parentModule)) {
            // this file fill be not cached, but it`s opener - will. And we have to remember it
            mockedModules[parent.filename] = true;
            mock.usedAs = mock.usedAs || [];
            mock.usedAs.push(baseRequest);

            mockedModules[baseRequest] = true;

            if (mock.allowCallThought) {
                if (!mock.original) {
                    mock.original = originalLoader(request, parent, isMain);
                }
            }

            if (mock.overrideBy) {
                if (!mock.override) {
                    if (typeof mock.overrideBy === 'string') {
                        mock.override = originalLoader(mock.overrideBy, parent, isMain);
                    } else {
                        mock.override = mock.overrideBy({
                            name: request,
                            fullName: baseRequest,
                            parent: parent,
                            original: mock.original
                        });
                    }
                }
                return mockResult(request, mock.override);
            }

            if (mock.allowCallThought) {
                if (typeof mock.original === 'function') {
                    if (_typeof(mock.value) === 'object' && Object.keys(mock.value).length === 0) {
                        return mockResult(request, mock.original);
                    } else {
                        throw new Error('rewiremock: trying to merge Functional base with CallThought mock at ' + request + '. Use overrideBy instead.');
                    }
                }
                return mockResult(request, Object.assign({}, mock.original, mock.value, {
                    __esModule: mock.original.__esModule
                }));
            }
            return mockResult(request, mock.value);
        } else {
            // why you shouldn't?
        }
    }

    if (isolation && !mockedModules[baseRequest]) {
        if (!testPassby(request, parent)) {
            throw new Error('rewiremock: isolation breach by [' + request + ']. Requested from ', parent.filename);
        }
    }

    return originalLoader(request, parent, isMain);
}

exports.default = mockLoader;