'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.cleanup = exports.removePlugin = exports.addPlugin = exports.mockModule = undefined;

var _module = require('module');

var _module2 = _interopRequireDefault(_module);

var _wipeCache = require('./wipeCache');

var _wipeCache2 = _interopRequireDefault(_wipeCache);

var _scope = require('./scope');

var _scope2 = _interopRequireDefault(_scope);

var _globals = require('./globals');

var _executor = require('./executor');

var _executor2 = _interopRequireDefault(_executor);

var _plugins = require('./plugins');

var _mocks = require('./mocks');

var _mock = require('./mock');

var _mock2 = _interopRequireDefault(_mock);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var parentModule = module.parent;
var mockScope = null;
var scope = function scope() {
    return (0, _globals.setScope)(mockScope);
};
var updateScope = function updateScope() {
    var parentScope = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;

    mockScope = (0, _scope2.default)(parentScope, parentModule);
    scope();
};

updateScope();

/** main **/

/**
 * @name rewiremock
 * @param {String} module name
 * @return {ModuleMock}
 */
function mockModule(moduleName) {
    scope();
    var name = (0, _plugins.convertName)(moduleName, parentModule);
    (0, _mocks.resetMock)(name);
    return (0, _plugins.onMockCreate)(new _mock2.default((0, _mocks.getMock)(name)));
}

/**
 * @name rewiremock.resolve
 * @param {String} module name
 * @return {String} converted module name
 */
mockModule.resolve = function (module) {
    scope();
    return (0, _plugins.convertName)(module, parentModule);
};

/** flags **/

/**
 * Activates module isolation
 * @param {Object} [options]
 * @param {Boolean} [options.noAutoPassBy] includes mocked modules to a isolation scope. Usage with mock.callThought.
 */
mockModule.isolation = function () {
    var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    mockScope.isolation = Object.assign({}, options);
};

/**
 * Deactivates isolation
 */
mockModule.withoutIsolation = function () {
    mockScope.isolation = false;
};

/**
 * Adding new passby record
 * @param {String|RegEx|Function} pattern
 */
mockModule.passBy = function (pattern) {
    mockScope.passBy.push(pattern);
};

mockModule.overrideEntryPoint = function (parent) {
    mockScope.parentModule = parentModule = parent || module.parent.parent;
};

/** interface **/

/**
 * enabled rewiremock
 */
mockModule.enable = function () {
    scope();
    _module2.default._load = _executor2.default;
    (0, _wipeCache2.default)();
    (0, _plugins.onEnable)((0, _mocks.getAllMocks)());
};

/**
 * disabled rewiremock
 */
mockModule.disable = function () {
    scope();
    _module2.default._load = _executor.originalLoader;
    (0, _plugins.onDisable)((0, _mocks.getAllMocks)());
    mockModule.withoutIsolation();
    mockModule.flush();
};

/**
 * Creates temporary executing scope. All mocks and plugins you will add in callback will be removed at exit.
 * @param callback
 */
mockModule.inScope = function (callback) {
    var currentScope = mockScope;
    var error = void 0;
    updateScope(currentScope);
    try {
        callback();
    } catch (e) {
        error = e;
    }

    mockScope = currentScope;
    if (error) throw error;
};

/**
 * executes module in sandbox
 * @param {Function} loader loader callback
 * @param {Function} [createCallback] - optional callback to be executed before load.
 * @return {Promise}
 */
mockModule.around = function (loader, createCallback) {
    return new Promise(function (resolve, reject) {
        var currentScope = mockScope;
        updateScope(currentScope);

        var restore = function restore() {
            mockModule.disable();
            mockScope = currentScope;
        };

        Promise.resolve(createCallback && createCallback(mockModule)).then(function () {
            return mockModule.enable();
        }).then(function () {
            return Promise.resolve(loader()).then(function (mockedResult) {
                restore();
                resolve(mockedResult);
            }, function (err) {
                restore();
                reject(err);
            });
        });
    });
};

/**
 * flushes all active overrides
 */
mockModule.flush = function () {
    (0, _wipeCache2.default)(mockScope.mockedModules);
    mockScope.mockedModules = {};
};

/**
 * flushes anything
 */
mockModule.clear = function () {
    updateScope();
    scope();
    mockModule.withoutIsolation();
    mockModule.flush();
};

var cleanup = function cleanup() {
    delete require.cache[require.resolve(__filename)];
};

var addPlugin = function addPlugin(plugin) {
    scope();
    (0, _plugins.addPlugin)(plugin);
};

var removePlugin = function removePlugin(plugin) {
    scope();
    (0, _plugins.removePlugin)(plugin);
};

mockModule.addPlugin = addPlugin;

exports.mockModule = mockModule;
exports.addPlugin = addPlugin;
exports.removePlugin = removePlugin;
exports.cleanup = cleanup;