'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.resetMock = exports.getAllMocks = exports.getMock = undefined;

var _path = require('path');

var _globals = require('./globals');

var _globals2 = _interopRequireDefault(_globals);

var _common = require('./_common');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var genMock = function genMock(name) {
    return {
        name: name,
        value: {}
    };
};

var resetMock = function resetMock(name) {
    return (0, _globals2.default)().mocks[name] = genMock(name);
};

var pickFrom = function pickFrom(mocks, name) {
    var ext = _common.extensions.find(function (ext) {
        return mocks[name + ext];
    });
    if (ext !== undefined) {
        return mocks[name + ext];
    }
};

var getMock = function getMock(name) {
    var scope = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : (0, _globals2.default)();
    var mocks = scope.mocks;

    var fn = (0, _path.parse)(name);
    var shortName = (0, _path.join)(fn.dir, fn.name);

    var mock = pickFrom(mocks, name) || pickFrom(mocks, shortName);

    if (!mock && scope.parentScope) {
        return getMock(name, scope.parentScope);
    }
    return mock;
};

var getAllMocks = function getAllMocks() {
    var result = {};
    var collect = function collect(scope) {
        if (scope.parentScope) {
            collect(scope.parentScope);
        }
        var mocks = scope.mocks;
        Object.keys(scope.mocks).forEach(function (key) {
            return result[key] = mocks[key];
        });
    };
    collect((0, _globals2.default)());
    return result;
};

exports.getMock = getMock;
exports.getAllMocks = getAllMocks;
exports.resetMock = resetMock;